package ru.redguy.extendedpistons;

import com.google.common.collect.Lists;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.*;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockPistonExtendEvent;
import org.bukkit.event.block.BlockPistonRetractEvent;
import org.bukkit.event.block.BlockRedstoneEvent;
import org.bukkit.material.PistonBaseMaterial;

import java.util.Comparator;
import java.util.*;

public class WorldListener implements Listener {
    @EventHandler(priority = EventPriority.LOW,ignoreCancelled = true)
    public void onPistonChanged(BlockPistonExtendEvent e) {
        if(e instanceof BlockPistonExtendedExtendEvent) return;
        Block block = e.getBlock();
        BlockState state = block.getState();
        PistonBaseMaterial piston = (PistonBaseMaterial) state.getData();
        int maxBlocks = findNearbyPlayerMaxBlocks(block.getLocation());

        ArrayList<Block> blocks = new ArrayList<>();
        Block toBreak = null;
        boolean isMovable = true;
        Block firstBlock = block.getRelative(piston.getFacing());
        if (firstBlock.isLiquid() || firstBlock.isEmpty()) {
        } else {
            for (int i = 1; i < maxBlocks + 1; i++) {
                Block b = block.getRelative(piston.getFacing(), i);
                if (b.isLiquid() || b.isEmpty()) break;
                if (b.getPistonMoveReaction().equals(PistonMoveReaction.IGNORE) || b.getPistonMoveReaction().equals(PistonMoveReaction.BLOCK)) {
                    if (!(b.getType() == Material.CHEST || b.getType() == Material.FURNACE || b.getType() == Material.BURNING_FURNACE)) {
                        isMovable = false;
                    } else {
                        blocks.add(b);
                    }
                } else if(b.getPistonMoveReaction().equals(PistonMoveReaction.BREAK)) {
                    toBreak = b;
                    break;
                } else {
                    blocks.add(b);
                }
            }
            Block lastBlock = block.getRelative(piston.getFacing(), blocks.size() + 1);
            if (!(lastBlock.getPistonMoveReaction().equals(PistonMoveReaction.BREAK) || lastBlock.isEmpty() || lastBlock.isLiquid()))
                isMovable = false;
        }

        if (!isMovable) {
            e.setCancelled(true);
        }
    }

    @EventHandler(priority = EventPriority.HIGHEST, ignoreCancelled = true)
    public void onRedstone(BlockRedstoneEvent e) {
        HashMap<BlockFace,BlockState> states = new HashMap<>();
        for (BlockFace face : new BlockFace[]{BlockFace.SELF, BlockFace.UP, BlockFace.DOWN, BlockFace.NORTH, BlockFace.SOUTH, BlockFace.EAST, BlockFace.WEST}) {
            states.put(face,e.getBlock().getRelative(face).getState());
        }
        Bukkit.getScheduler().runTask(ExtendedPistons.INSTANCE,() -> {
            if (e.getOldCurrent() != 0 && e.getNewCurrent() == 0) {
                ArrayList<BlockFace> checkedBlockFaces = new ArrayList<BlockFace>() {{
                    add(BlockFace.SELF);
                    add(BlockFace.UP);
                    add(BlockFace.DOWN);
                }};
                if (e.getBlock().getType().equals(Material.REDSTONE_WIRE)) {
                    for (BlockFace blockFace : new BlockFace[]{BlockFace.NORTH, BlockFace.SOUTH, BlockFace.EAST, BlockFace.WEST}) {
                        Block block = e.getBlock().getRelative(blockFace);
                        if (block.getBlockPower() == 0) checkedBlockFaces.add(blockFace);
                    }
                } else {
                    checkedBlockFaces.add(BlockFace.NORTH);
                    checkedBlockFaces.add(BlockFace.SOUTH);
                    checkedBlockFaces.add(BlockFace.EAST);
                    checkedBlockFaces.add(BlockFace.WEST);
                }
                for (BlockFace face : checkedBlockFaces) {
                    Block block = e.getBlock().getRelative(face);
                    BlockState state = states.get(face);
                    if (state.getType().equals(Material.PISTON_STICKY_BASE)) {
                        PistonBaseMaterial piston = (PistonBaseMaterial) state.getData();

                        Block b = block.getRelative(piston.getFacing(),2);
                        if(b.getType().equals(Material.CHEST)||b.getType().equals(Material.FURNACE)||b.getType().equals(Material.BURNING_FURNACE)) {
                            Block b2 = block.getRelative(piston.getFacing());
                            BlockPistonRetractEvent ev = new BlockPistonRetractEvent(block, Lists.newArrayList(b2),piston.getFacing());
                            Bukkit.getPluginManager().callEvent(ev);
                            if(ev.isCancelled()) continue;
                                b2.setType(b.getType());
                                b2.setData(b.getData());
                                if (b.getType().equals(Material.CHEST)) {
                                    Chest oldChest = (Chest) b.getState();
                                    Chest newChest = (Chest) b2.getState();
                                    newChest.getBlockInventory().setContents(oldChest.getBlockInventory().getContents());
                                    oldChest.getBlockInventory().clear();
                                } else if (b.getType().equals(Material.FURNACE) || b.getType().equals(Material.BURNING_FURNACE)) {
                                    Furnace oldFurnace = (Furnace) b.getState();
                                    Furnace newFurnace = (Furnace) b2.getState();
                                    newFurnace.getInventory().setContents(oldFurnace.getInventory().getContents());
                                    oldFurnace.getInventory().clear();
                                }
                                b.setType(Material.AIR);
                        }
                    }
                }
            } else {
                ArrayList<BlockFace> checkedBlockFaces = new ArrayList<BlockFace>() {{
                    add(BlockFace.SELF);
                    add(BlockFace.UP);
                    add(BlockFace.DOWN);
                }};
                if (e.getBlock().getType().equals(Material.REDSTONE_WIRE)) {
                    for (BlockFace blockFace : new BlockFace[]{BlockFace.NORTH, BlockFace.SOUTH, BlockFace.EAST, BlockFace.WEST}) {
                        Block block = e.getBlock().getRelative(blockFace);
                        if (block.getBlockPower() != 0) checkedBlockFaces.add(blockFace);
                    }
                } else {
                    checkedBlockFaces.add(BlockFace.NORTH);
                    checkedBlockFaces.add(BlockFace.SOUTH);
                    checkedBlockFaces.add(BlockFace.EAST);
                    checkedBlockFaces.add(BlockFace.WEST);
                }
                for (BlockFace face : checkedBlockFaces) {
                    Block block = e.getBlock().getRelative(face);
                    if (block.getType().equals(Material.PISTON_BASE) || block.getType().equals(Material.PISTON_STICKY_BASE)) {
                        BlockState state = states.get(face);
                        PistonBaseMaterial piston = (PistonBaseMaterial) state.getData();
                        int maxBlocks = findNearbyPlayerMaxBlocks(block.getLocation());

                        ArrayList<Block> blocks = new ArrayList<>();
                        Block toBreak = null;
                        boolean isMovable = true;
                        Block firstBlock = block.getRelative(piston.getFacing());
                        if (firstBlock.isLiquid() || firstBlock.isEmpty()) {
                        } else {
                            for (int i = 1; i < maxBlocks + 1; i++) {
                                Block b = block.getRelative(piston.getFacing(), i);
                                if (b.isLiquid() || b.isEmpty()) break;
                                if (b.getPistonMoveReaction().equals(PistonMoveReaction.IGNORE) || b.getPistonMoveReaction().equals(PistonMoveReaction.BLOCK)) {
                                    if (!(b.getType() == Material.CHEST || b.getType() == Material.FURNACE || b.getType() == Material.BURNING_FURNACE)) {
                                        isMovable = false;
                                    } else {
                                        blocks.add(b);
                                    }
                                } else if (b.getPistonMoveReaction().equals(PistonMoveReaction.BREAK)) {
                                    toBreak = b;
                                    break;
                                } else {
                                    blocks.add(b);
                                }
                            }
                            Block lastBlock = block.getRelative(piston.getFacing(), blocks.size() + 1);
                            if (!(lastBlock.getPistonMoveReaction().equals(PistonMoveReaction.BREAK) || lastBlock.isEmpty() || lastBlock.isLiquid()))
                                isMovable = false;
                        }


                        if (isMovable) {
                            BlockPistonExtendedExtendEvent event = new BlockPistonExtendedExtendEvent(block, blocks, piston.getFacing());
                            Bukkit.getPluginManager().callEvent(event);
                            if (event.isCancelled()) return;
                            Collections.reverse(blocks);
                            if (toBreak != null) toBreak.breakNaturally();
                            for (Block b : blocks) {
                                Block b2 = b.getRelative(piston.getFacing());
                                b2.setType(b.getType());
                                b2.setData(b.getData());
                                if (b.getType().equals(Material.CHEST)) {
                                    Chest oldChest = (Chest) b.getState();
                                    Chest newChest = (Chest) b2.getState();
                                    newChest.getBlockInventory().setContents(oldChest.getBlockInventory().getContents());
                                    oldChest.getBlockInventory().clear();
                                } else if (b.getType().equals(Material.FURNACE) || b.getType().equals(Material.BURNING_FURNACE)) {
                                    Furnace oldFurnace = (Furnace) b.getState();
                                    Furnace newFurnace = (Furnace) b2.getState();
                                    newFurnace.getInventory().setContents(oldFurnace.getInventory().getContents());
                                    oldFurnace.getInventory().clear();
                                }
                                b.setType(Material.AIR);
                            }
                        }
                    }
                }
            }
        });
    }

    public int findNearbyPlayerMaxBlocks(Location location) {
        Optional<? extends Player> op = Bukkit.getOnlinePlayers().stream().min(Comparator.comparingDouble(p -> p.getLocation().distance(location)));
        if(op.isPresent()) {
            String[] groups = ExtendedPistons.INSTANCE.permsService.getPlayerGroups(op.get());
            for (String group : groups) {
                if(ExtendedPistons.INSTANCE.conf.isSet("piston-move-limitations."+group)&&ExtendedPistons.INSTANCE.conf.isInt("piston-move-limitations."+group))
                    return ExtendedPistons.INSTANCE.conf.getInt("piston-move-limitations."+group);
            }
        }
        return 12;
    }
}
